package com.sky.controller.admin;


import com.sky.common.result.PageResult;
import com.sky.common.result.Result;
import com.sky.common.utils.RedisUtil;
import com.sky.pojo.dto.DishDTO;
import com.sky.pojo.dto.DishPageQueryDTO;
import com.sky.pojo.entity.Dish;
import com.sky.pojo.vo.DishVO;
import com.sky.service.DishService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequiredArgsConstructor
@RequestMapping("/admin/dish")
@Api(tags = "菜品相关接口")
@Slf4j
public class DishController {
    private final DishService dishService;

    @PostMapping
    @ApiOperation("新增菜品")
    public Result<DishDTO> save(@RequestBody DishDTO dishDTO) {
        log.info("新增菜品:{}", dishDTO);
        // 删除redis中缓存
        RedisUtil.cleanCache("dish_" + dishDTO.getCategoryId());
        dishService.saveWithFlavor(dishDTO);
        return Result.success();
    }

    @GetMapping("/page")
    @ApiOperation("菜品分页查询")
    public Result<PageResult<DishVO>> page(DishPageQueryDTO dishPageQueryDTO) {
        log.info("菜品分页查询:{}", dishPageQueryDTO);
        PageResult<DishVO> pageResult = dishService.pageQuery(dishPageQueryDTO);
        return Result.success(pageResult);
    }

    @DeleteMapping
    @ApiOperation("菜品批量删除")
    public Result<String> delete(@RequestParam List<Long> ids) {
        log.info("菜品批量删除:{}", ids);
        dishService.deleteBatch(ids);
        // 将菜品数据删除
        RedisUtil.cleanCache("dish_*");
        return Result.success();
    }

    @GetMapping("/{id}")
    @ApiOperation("根据id查询菜品和对应的口味数据")
    public Result<DishVO> getById(@PathVariable Long id) {
        log.info("根据id查询菜品和对应的口味数据：{}", id);
        DishVO dishVO = dishService.getById(id);
        return Result.success(dishVO);
    }

    @PutMapping
    @ApiOperation("修改菜品")
    public Result<DishDTO> update(@RequestBody DishDTO dto) {
        log.info("修改菜品：{}", dto);
        dishService.updateWithFlavor(dto);
        // 将菜品数据删除
        RedisUtil.cleanCache("dish_*");
        return Result.success();
    }

    @PostMapping("status/{status}")
    @ApiOperation("菜品起售、停售")
    public Result<String> status(@PathVariable Integer status, Long id) {
        log.info("菜品起售、停售：{}，菜品id：{}", status, id);
        dishService.updateWithStatus(status, id);
        // 将菜品数据删除
        RedisUtil.cleanCache("dish_*");
        return Result.success();
    }

    @GetMapping("list")
    @ApiOperation("根据分类id查询菜品")
    public Result<List<Dish>> list(Long categoryId) {
        log.info("根据分类id查询菜品：{}", categoryId);
        List<Dish> dishVOList = dishService.getByCategoryId(categoryId);
        return Result.success(dishVOList);
    }
}
