package com.sky.service.impl;

import com.sky.mapper.OrderMapper;
import com.sky.pojo.dto.GoodsSalesDTO;
import com.sky.pojo.entity.Orders;
import com.sky.pojo.vo.*;
import com.sky.service.ReportService;
import com.sky.service.WorkspaceService;
import com.sky.service.utils.ReportServiceImplUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class ReportServiceImpl implements ReportService {
    @Autowired
    ReportServiceImplUtils reportServiceImplUtils;
    @Autowired
    WorkspaceService workspaceService;
    @Autowired
    private OrderMapper orderMapper;

    /**
     * 营业额数据统计
     *
     * @param begin 开始时间
     * @param end   结束时间
     * @return 营业额数据统计
     */
    @Override
    public TurnoverReportVO getTurnover(LocalDate begin, LocalDate end) {
        ArrayList<LocalDate> dateArrayList = new ArrayList<>();
        List<Double> turnoverList = new ArrayList<>();

        dateArrayList.add(begin);
        while (!begin.equals(end)) {
            // 计算指定日期的后一天
            begin = begin.plusDays(1);
            dateArrayList.add(begin);
        }

        // 查询数据
        for (LocalDate date : dateArrayList) {
            // 查询date日期对应营业额数据
            LocalDateTime beginTime = LocalDateTime.of(date, LocalTime.MIN);
            LocalDateTime endTime = LocalDateTime.of(date, LocalTime.MAX);
            Map map = new HashMap();
            map.put("status", Orders.COMPLETED);
            map.put("begin", beginTime);
            map.put("end", endTime);
            // 根据动态条件统计营业额
            Double turnover = orderMapper.sumByMap(map);
            turnover = turnover == null ? 0.0 : turnover;
            turnoverList.add(turnover);
        }

        return TurnoverReportVO.builder()
                // 将 dateArrayList 变成字符串
                .dateList(StringUtils.join(dateArrayList, "，"))
                // 将数据变成字符串
                .turnoverList(StringUtils.join(turnoverList, ","))
                .build();
    }

    /**
     * 用户数据统计
     *
     * @param begin 开始时间
     * @param end   结束时间
     * @return 用户数据统计
     */
    @Override
    public UserReportVO getUserStatistics(LocalDate begin, LocalDate end) {
        // 初始化时间
        ArrayList<LocalDate> dateArrayList = new ArrayList<>();
        // 新增用户数
        ArrayList<Integer> newUserList = new ArrayList<>();
        // 总用户数
        ArrayList<Integer> totalUserList = new ArrayList<>();

        dateArrayList.add(begin);
        while (!begin.equals(end)) {
            begin = begin.plusDays(1);
            dateArrayList.add(begin);
        }

        for (LocalDate date : dateArrayList) {
            LocalDateTime beginTime = LocalDateTime.of(date, LocalTime.MIN);
            LocalDateTime endTime = LocalDateTime.of(date, LocalTime.MAX);
            // 新增用户数
            Integer newUser = reportServiceImplUtils.getUserCount(beginTime, endTime);
            // 总用户数
            Integer totalUser = reportServiceImplUtils.getUserCount(null, endTime);
            newUserList.add(newUser);
            totalUserList.add(totalUser);
        }

        return UserReportVO.builder()
                .dateList(StringUtils.join(dateArrayList, ","))
                .newUserList(StringUtils.join(newUserList, ","))
                .totalUserList(StringUtils.join(totalUserList, ","))
                .build();
    }

    /**
     * 订单统计接口
     *
     * @param begin 开始时间
     * @param end   结束时间
     * @return 订单统计
     */
    @Override
    public OrderReportVO getOrdersStatistics(LocalDate begin, LocalDate end) {
        // 封装dateList
        ArrayList<LocalDate> dateList = new ArrayList<>();
        dateList.add(begin);
        while (!begin.equals(end)) {
            begin = begin.plusDays(1);
            dateList.add(begin);
        }

        // 查询每天订单总数 & 查询每天有效订单数
        ArrayList<Integer> orderCountList = new ArrayList<>();
        ArrayList<Integer> validOrderCountList = new ArrayList<>();
        for (LocalDate date : dateList) {
            LocalDateTime beginTime = LocalDateTime.of(date, LocalTime.MIN);
            LocalDateTime endTime = LocalDateTime.of(date, LocalTime.MAX);
            // 查询每天订单总数
            Integer orderCount = reportServiceImplUtils.getOrderCount(beginTime, endTime, null);
            orderCountList.add(orderCount);
            // 查询每天有效订单数
            Integer validOrderCount = reportServiceImplUtils.getOrderCount(beginTime, endTime, 5);
            validOrderCountList.add(validOrderCount);
        }

        // 订单总数
        Integer totalOrderCount = orderCountList.stream().reduce(Integer::sum).orElse(0);// 如果为空，则返回默认值 0
        // 有效订单数
        Integer validOrderCount = validOrderCountList.stream().reduce(Integer::sum).orElse(0);// 如果为空，则返回默认值 0
        // 订单完成率
        double orderCompletionRate = totalOrderCount != 0 ? validOrderCount.doubleValue() / totalOrderCount.doubleValue() : 0.0;

        return OrderReportVO.builder()
                .dateList(StringUtils.join(dateList, ","))
                .orderCountList(StringUtils.join(orderCountList, ","))
                .validOrderCountList(StringUtils.join(validOrderCountList, ","))
                .totalOrderCount(totalOrderCount)
                .validOrderCount(validOrderCount)
                .orderCompletionRate(orderCompletionRate).build();
    }

    /**
     * 查询销量排名top10接口
     *
     * @param begin 开始时间
     * @param end   结束时间
     * @return 销量排名top10
     */
    @Override
    public SalesTop10ReportVO getTop10Statistics(LocalDate begin, LocalDate end) {
        // 查询销量前10列表
        LocalDateTime beginTime = LocalDateTime.of(begin, LocalTime.MIN);
        LocalDateTime endTime = LocalDateTime.of(end, LocalTime.MAX);
        List<GoodsSalesDTO> salesTop = orderMapper.getSalesTop10(beginTime, endTime);
        List<String> nameList = salesTop.stream().map(GoodsSalesDTO::getName).collect(Collectors.toList());
        List<Integer> numberList = salesTop.stream().map(GoodsSalesDTO::getNumber).collect(Collectors.toList());
        return SalesTop10ReportVO.builder()
                .nameList(StringUtils.join(nameList, ","))
                .numberList(StringUtils.join(numberList, ",")).build();
    }

    /**
     * 导出Excel报表接口
     *
     * @param response HttpServletResponse
     */
    @Override
    public void exportBusinessData(HttpServletResponse response) {
        // 1. 查询数据库，获取营业数据
        LocalDate dateBegin = LocalDate.now().minusDays(30);
        LocalDate dateEnd = LocalDate.now().minusDays(1);
        // 查询概览数据
        BusinessDataVO businessDataVO = workspaceService.getBusinessData(LocalDateTime.of(dateBegin, LocalTime.MIN), LocalDateTime.of(dateEnd, LocalTime.MAX));
        // 2. 通过POI将数据写入到Excel文件中
        InputStream in = this.getClass().getClassLoader().getResourceAsStream("template/运营数据报表模板.xlsx");
        try {
            XSSFWorkbook excel = new XSSFWorkbook(in);
            XSSFSheet sheet = excel.getSheet("Sheet1");
            sheet.getRow(1).getCell(1).setCellValue("时间" + dateBegin + "至" + dateEnd);// 标题
            sheet.getRow(3).getCell(2).setCellValue(businessDataVO.getTurnover());// 营业额
            sheet.getRow(3).getCell(4).setCellValue(businessDataVO.getOrderCompletionRate());// 订单完成率
            sheet.getRow(3).getCell(6).setCellValue(businessDataVO.getNewUsers());// 新增用户数
            sheet.getRow(4).getCell(2).setCellValue(businessDataVO.getValidOrderCount());// 有效订单数
            sheet.getRow(4).getCell(4).setCellValue(businessDataVO.getUnitPrice());// 平均客单价

            for (int i = 0; i < 30; i++) {
                LocalDate date = dateBegin.plusDays(i);
                // 查询某一天数据
                BusinessDataVO businessData = workspaceService.getBusinessData(LocalDateTime.of(date, LocalTime.MIN), LocalDateTime.of(date, LocalTime.MAX));
                XSSFRow row = sheet.getRow(7 + i);
                row.getCell(1).setCellValue(date.toString());
                row.getCell(2).setCellValue(businessData.getTurnover());
                row.getCell(3).setCellValue(businessData.getValidOrderCount());
                row.getCell(4).setCellValue(businessData.getOrderCompletionRate());
                row.getCell(5).setCellValue(businessData.getUnitPrice());
                row.getCell(6).setCellValue(businessData.getNewUsers());
            }
            // 3. 通过输出流将Excel下载到客户端浏览器
            ServletOutputStream out = response.getOutputStream();
            excel.write(out);
            excel.close();
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
