package com.sky.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.sky.common.constant.MessageConstant;
import com.sky.common.constant.StatusConstant;
import com.sky.common.exception.SetMealEnableFailedException;
import com.sky.common.result.PageResult;
import com.sky.mapper.DishMapper;
import com.sky.mapper.SetMealDishMapper;
import com.sky.mapper.SetmealMapper;
import com.sky.pojo.dto.SetmealDTO;
import com.sky.pojo.dto.SetmealPageQueryDTO;
import com.sky.pojo.entity.Dish;
import com.sky.pojo.entity.Setmeal;
import com.sky.pojo.entity.SetmealDish;
import com.sky.pojo.vo.DishItemVO;
import com.sky.pojo.vo.SetmealVO;
import com.sky.service.SetmealService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@Transactional
public class SetmealServiceImpl implements SetmealService {
    @Autowired
    SetmealMapper setmealMapper;
    @Autowired
    SetMealDishMapper setMealDishMapper;
    @Autowired
    DishMapper dishMapper;

    /**
     * 分页查询
     *
     * @param dto 前端请求
     * @return PageResult
     */
    @Override
    public PageResult<SetmealVO> pageQuery(SetmealPageQueryDTO dto) {
        int pageSize = dto.getPageSize();
        int page = dto.getPage();
        PageHelper.startPage(page, pageSize);

        Page<SetmealVO> setmealVOPage = setmealMapper.pageQuery(dto);
        return new PageResult<>(setmealVOPage.getTotal(), setmealVOPage.getResult());
    }

    /**
     * 根据id查询套餐
     *
     * @param id 套餐id
     * @return 套餐相关信息
     */
    @Override
    public SetmealVO page4Id(Long id) {
        // 查找套餐
        Setmeal setmeal = setmealMapper.getById(id);
        // 根据id查找菜品
        List<SetmealDish> setmealDishList = setMealDishMapper.getBySetmealId(id);
        SetmealVO setmealVO = new SetmealVO();

        BeanUtils.copyProperties(setmeal, setmealVO);
        setmealVO.setSetmealDishes(setmealDishList);

        return setmealVO;
    }

    /**
     * 修改套餐
     *
     * @param dto 前端请求
     */
    @Override
    public void update(SetmealDTO dto) {
        Setmeal setmeal = new Setmeal();
        BeanUtils.copyProperties(dto, setmeal);
        // 修改套餐
        setmealMapper.update(setmeal);

        // 先删除相关套餐菜品
        Long setmealId = dto.getId();
        setMealDishMapper.deleteBySetmealId(setmealId);

        // 插入菜品
        List<SetmealDish> setmealDishes = dto.getSetmealDishes();
        setmealDishes.forEach(setmealDish -> setmealDish.setSetmealId(setmealId));
        setMealDishMapper.insert(setmealDishes);
    }

    /**
     * 新增套餐
     *
     * @param setmealDTO 套餐信息
     */
    @Override
    @Transactional
    public void insert(SetmealDTO setmealDTO) {
        // 插入套餐
        Setmeal setmeal = new Setmeal();
        BeanUtils.copyProperties(setmealDTO, setmeal);
        setmealMapper.insert(setmeal);

        // 插入套餐菜品
        List<SetmealDish> setmealDishes = setmealDTO.getSetmealDishes();
        setmealDishes.forEach(setmealDish -> setmealDish.setSetmealId(setmeal.getId()));

        setMealDishMapper.insert(setmealDishes);
    }


    /**
     * 套餐起售、停售
     *
     * @param status 状态
     * @param id     id
     */
    @Override
    public void updateStatus(Integer status, Long id) {
        List<Dish> byCategoryId = dishMapper.getBySetmealId(id);
        // 判断当前是否有停售的菜品，如果有则不能起售
        if (status.equals(StatusConstant.ENABLE)) {
            byCategoryId.forEach(dishVO -> {
                if (dishVO.getStatus().equals(StatusConstant.DISABLE)) {
                    throw new SetMealEnableFailedException(MessageConstant.SETMEAL_ENABLE_FAILED);
                }
            });
        }
        // 修改起售或停售
        Setmeal setmeal = Setmeal.builder().status(status).id(id).build();
        setmealMapper.update(setmeal);
    }

    /**
     * 批量删除套餐
     *
     * @param ids 要删除套餐ids
     */
    @Override
    @Transactional
    public void delete(List<Long> ids) {
        // 查看当前是否是起售，起售中不能删除
        ids.forEach(id -> {
            Setmeal setmeal = setmealMapper.getById(id);
            if (setmeal.getStatus().equals(StatusConstant.ENABLE)) {
                throw new SetMealEnableFailedException(MessageConstant.SETMEAL_ON_SALE);
            }
        });
        setmealMapper.delete(ids);
        setMealDishMapper.deleteBySetmealIds(ids);
    }

    /**
     * 条件查询
     *
     * @param setmeal 套餐
     * @return List<Setmeal>
     */
    public List<Setmeal> list(Setmeal setmeal) {
        return setmealMapper.list(setmeal);
    }

    /**
     * 根据id查询菜品选项
     *
     * @param id id
     * @return List<DishItemVO>
     */
    public List<DishItemVO> getDishItemById(Long id) {
        return setmealMapper.getDishItemBySetmealId(id);
    }
}
